/* Copyright (C) 2000 Ross Diankov

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Name:	object3d.h
// Purpose: Organizes 3d data in structures and provides the basic
//          functions to initiate and transform 3d objects.

#ifndef OBJECT3D_CLASSES
#define OBJECT3D_CLASSES

// vertex contains information like color, position, etc.
struct VERTEX3D
{
	VERTEX3D() {}
	VERTEX3D(VECTOR& pos, VECTOR& normal, SCALAR tu = 0.0f, SCALAR tv = 0.0f);		// default constructor
	VECTOR pos;			// position	
						// normal of the vertex calculated from the average of the normals of the faces
	VECTOR normal;		// a vertex normal for gouraud shading
	SCALAR u, v;		// texture coordinates
	SCALAR color;		// color intensity of vertex
};

struct OBJECT3D
{
	OBJECT3D(int numverts, int numfaces);
	~OBJECT3D() { delete faces; delete vertices; }

	WORD* faces;					// pointer to faces
	VERTEX3D* vertices;				// array of vertices
	VECTOR* face_normals;			// array of normals of the faces = (numFaces / 3);

	int numVertices;
	int numFaces;
};

struct COLOR3D
{
	SCALAR r, g, b;
};

struct LIGHT3D
{
	VECTOR pos;						// the position of the light
									// not aplicable for ambient light
	VECTOR dir;						// direction where the light is pointing
									// not applicable for point or ambient light
	SCALAR att;						// fall-off factor
	COLOR3D color;					// color of the light
};

struct MATERIAL3D
{
	COLOR3D ambient;				// RGB values that range from 0.0 to 1.0
	COLOR3D diffuse;
	COLOR3D specular;
	SCALAR exp;						// specular exponent  cos^exp
};

struct TEXTURE3D
{
	DWORD dwWidth, dwHeight;		// the dimensions of the texture
	DWORD dwPitch;					// the size in bytes of one scanline
	LPBYTE bytes;					// a pointer to memory of the texture information
	HANDLE hBitmap;					// handle to the bitmap
};

void CreateTexture(char *, TEXTURE3D**);
void DestroyTexture(TEXTURE3D*);

void SetProjectionMatrix(MATRIX&, SCALAR fFOVx, SCALAR fFOVy, SCALAR fAspect, 
						 SCALAR fNearPlane, SCALAR fFarPlane);
void SetViewMatrix(MATRIX&, VECTOR& vPos, VECTOR& vLookAt, VECTOR& vUp);
void VectorInterpolate(VECTOR&, VECTOR&, VECTOR&, SCALAR);
void SetRotationMatrix( MATRIX& mat, VECTOR& vDir, SCALAR fRads);
void ComputeObjectNormals(OBJECT3D*); 

#endif